
module sample_top (

	input 				clk_148_p,				
	input 				clk_125_p,				
	output 				sdi_tx_p,					
	input 				sdi_rx_p,					
	output				sdi_rx_bypass,				
	output				sdi_tx_sd_hdn,				
	output				sdi_tx_en,					
	output				sdi_rx_en,					
	output rx_frame_locked, //R7
	output rx_trs_locked, //R6
	input		[7:0] 	user_dipsw,	
	input 				cpu_resetn,			
	output	[19:0]	rx_dataout
);

assign	sdi_tx_sd_hdn              = 0;	// HD Mode Enable
assign	sdi_tx_en	               = 1;	// Transmit Enable
assign	sdi_rx_en	               = 1'bz; // Mute pin tied through R118=0 Ohm = CD* Receive Enable - Tri-state
assign	sdi_rx_bypass              = 0;	// Receive Bypass	 
	
wire				tx_clkout;
wire 	[139:0] 	reconfig_to_xcvr;
wire 	[91:0] 	reconfig_from_xcvr;
wire				rx_sdi_start_reconfig;
wire	[1:0]		rx_sdi_reconfig_done;
wire	[1:0]		sdi_rx_std;
wire  			reconfig_busy;
wire  [8:0] 	reconfig_mgmt_address;
wire    			reconfig_mgmt_read;
wire  [31:0] 	reconfig_mgmt_readdata;
wire    			reconfig_mgmt_waitrequest;
wire    			reconfig_mgmt_write;
wire  [31:0] 	reconfig_mgmt_writedata;
wire  [31:0] 	reconfig_mif_address;
wire    			reconfig_mif_read;
wire 	[15:0] 	reconfig_mif_readdata;
wire    			reconfig_mif_waitrequest;

wire rx_frame;
wire rx_trs;
wire tx_dataout_valid;

assign rx_frame_locked = ~rx_frame;
assign rx_trs_locked = ~rx_trs;

wire [19:0] pattgen_dout_export;
wire pattgen_dout_valid_export;
wire pattgen_trs_export;
	
wire [19:0] sdi_tx_data_in;
wire sdi_tx_valid_in;
wire sdi_tx_trs_in;
	

wire [19:0]	cvo_vid_data;
wire [10:0]	cvo_vid_ln;
wire cvo_underflow,cvo_vid_trs;
wire vid_clk;
wire [1:0] vid_std;
wire vid_mode_change;	
	
vip_system vip_instance (
	. clk_clk(clk_148_p), 
	. reset_reset_n (cpu_resetn),
	. cvo_vid_clk (vid_clk),
	. cvo_vid_data (cvo_vid_data),
	. cvo_underflow (cvo_underflow),
	. cvo_vid_trs (cvo_vid_trs),
	. cvo_vid_ln (cvo_vid_ln),
	. cvo_vid_std (vid_std),
	. cvo_vid_mode_change(vid_mode_change)
);
	
assign     vid_clk = vid_std[1] ? tx_clkout : (vid_std[0] ? tx_dataout_valid : tx_dataout_valid);

assign sdi_tx_data_in = cvo_vid_data;
assign sdi_tx_valid_in = tx_dataout_valid;
assign sdi_tx_trs_in = cvo_vid_trs;
	
	
SDI_II_duplex sdi_ii_dup_inst
(
	.rx_rst								(!cpu_resetn) ,	// input  rx_rst_sig
	.tx_rst								(!cpu_resetn) ,	// input  tx_rst_sig
	.rx_coreclk							(clk_148_p) ,	// input  rx_coreclk_sig
	.tx_pclk								(tx_clkout) ,	// input  tx_pclk_sig
	.tx_coreclk							(clk_148_p) ,	// input  tx_coreclk_sig
	.xcvr_refclk						(clk_148_p) ,	// input  xcvr_refclk_sig
	.tx_clkout							(tx_clkout) ,	// output  tx_clkout_sig
	.tx_enable_crc						() ,	// input  tx_enable_crc_sig
	.tx_enable_ln						(1'b1) ,	// input  tx_enable_ln_sig
	.tx_ln								(cvo_vid_ln) ,	// input [10:0] tx_ln_sig
	.tx_std								(vid_std) ,	// input [1:0] tx_std_sig
	.tx_datain							(sdi_tx_data_in) ,	// input [19:0] tx_datain_sig
	.tx_datain_valid					(tx_dataout_valid) ,	// input  tx_datain_valid_sig
	.tx_trs								(cvo_vid_trs) ,	// input  tx_trs_sig
	.tx_ln_b								() ,	// input [10:0] tx_ln_b_sig
	.tx_dataout_valid					(tx_dataout_valid) ,	// output  tx_dataout_valid_sig
	.tx_pll_locked						() ,	// output  tx_pll_locked_sig
   .rx_clkout							() ,	// output  rx_clkout_sig
	.rx_dataout							(rx_dataout) ,	// output [19:0] rx_dataout_sig
	.rx_dataout_valid					() ,	// output  rx_dataout_valid_sig
	.rx_f									() ,	// output  rx_f_sig
	.rx_v									() ,	// output  rx_v_sig
	.rx_h									() ,	// output  rx_h_sig
	.rx_ap								() ,	// output  rx_ap_sig
	.rx_format							() ,	// output [3:0] rx_format_sig
	.rx_eav								() ,	// output  rx_eav_sig
	.rx_trs								() ,	// output  rx_trs_sig
	.rx_std								(sdi_rx_std) ,	// output [1:0] rx_std_sig
	.rx_align_locked					() ,	// output  rx_align_locked_sig
	.rx_trs_locked						(rx_trs) ,	// output  rx_trs_locked_sig
	.rx_frame_locked					(rx_frame) ,	// output  rx_frame_locked_sig
	.rx_pll_locked						() ,	// output  rx_pll_locked_sig
	.rx_ln								() ,	// output [10:0] rx_ln_sig
	.rx_ln_b								() ,	// output [10:0] rx_ln_b_sig
	.rx_vpid_byte1						() ,	// output [7:0] rx_vpid_byte1_sig
	.rx_vpid_byte2						() ,	// output [7:0] rx_vpid_byte2_sig
	.rx_vpid_byte3						() ,	// output [7:0] rx_vpid_byte3_sig
	.rx_vpid_byte4						() ,	// output [7:0] rx_vpid_byte4_sig
	.rx_vpid_valid						() ,	// output  rx_vpid_valid_sig
	.rx_vpid_checksum_error			() ,	// output  rx_vpid_checksum_error_sig
	.rx_line_f0							() ,	// output [10:0] rx_line_f0_sig
	.rx_line_f1							() ,	// output [10:0] rx_line_f1_sig
	.rx_vpid_byte1_b					() ,	// output [7:0] rx_vpid_byte1_b_sig
	.rx_vpid_byte2_b					() ,	// output [7:0] rx_vpid_byte2_b_sig
	.rx_vpid_byte3_b					() ,	// output [7:0] rx_vpid_byte3_b_sig
	.rx_vpid_byte4_b					() ,	// output [7:0] rx_vpid_byte4_b_sig
	.rx_vpid_valid_b					() ,	// output  rx_vpid_valid_b_sig
	.rx_vpid_checksum_error_b		() ,	// output  rx_vpid_checksum_error_b_sig
	.rx_rst_proto_out					() ,	// output  rx_rst_proto_out_sig
	.rx_coreclk_is_ntsc_paln		(1'b1) ,	// input  rx_coreclk_is_ntsc_paln_sig
	.rx_clkout_is_ntsc_paln			() ,	// output  rx_clkout_is_ntsc_paln_sig
	.rx_sdi_start_reconfig			(rx_sdi_start_reconfig) ,	// output  rx_sdi_start_reconfig_sig
	.rx_sdi_reconfig_done			(rx_sdi_reconfig_done[0]) ,	// input  rx_sdi_reconfig_done_sig
	.sdi_tx								(sdi_tx_p) ,	// output  sdi_tx_sig
	.sdi_rx								(sdi_rx_p) ,	// input  sdi_rx_sig
	.reconfig_to_xcvr					(reconfig_to_xcvr) ,	// input [139:0] reconfig_to_xcvr_sig
	.reconfig_from_xcvr				(reconfig_from_xcvr) 	// output [91:0] reconfig_from_xcvr_sig
);


sdi_ii_ed_reconfig_mgmt sdi_ii_ed_reconfig_mgmt_inst
(
	.reconfig_clk						(clk_125_p) ,	// input  reconfig_clk_sig
	.rst									(!cpu_resetn) ,	// input  rst_sig
	.sdi_rx_start_reconfig			({1'b0,rx_sdi_start_reconfig}) ,	// input [NUM_CHS-1:0] sdi_rx_start_reconfig_sig
	.sdi_rx_std							({2'b00,sdi_rx_std}) ,	// input [NUM_CHS*2-1:0] sdi_rx_std_sig
	.sdi_rx_reconfig_done			(rx_sdi_reconfig_done[0]) ,	// output [NUM_CHS-1:0] sdi_rx_reconfig_done_sig
	.reconfig_busy						(reconfig_busy) ,	// input  reconfig_busy_sig
	.reconfig_mgmt_address			(reconfig_mgmt_address) ,	// output [8:0] reconfig_mgmt_address_sig
	.reconfig_mgmt_read				(reconfig_mgmt_read) ,	// output  reconfig_mgmt_read_sig
	.reconfig_mgmt_readdata			(reconfig_mgmt_readdata) ,	// input [31:0] reconfig_mgmt_readdata_sig
	.reconfig_mgmt_waitrequest		(reconfig_mgmt_waitrequest) ,	// input  reconfig_mgmt_waitrequest_sig
	.reconfig_mgmt_write				(reconfig_mgmt_write) ,	// output  reconfig_mgmt_write_sig
	.reconfig_mgmt_writedata		(reconfig_mgmt_writedata) ,	// output [31:0] reconfig_mgmt_writedata_sig
	.reconfig_mif_address			(reconfig_mif_address) ,	// input [31:0] reconfig_mif_address_sig
	.reconfig_mif_read				(reconfig_mif_read) ,	// input  reconfig_mif_read_sig
	.reconfig_mif_readdata			(reconfig_mif_readdata) ,	// output [15:0] reconfig_mif_readdata_sig
	.reconfig_mif_waitrequest		(reconfig_mif_waitrequest) 	// output  reconfig_mif_waitrequest_sig
);

defparam sdi_ii_ed_reconfig_mgmt_inst.NUM_CHS = 2;
defparam sdi_ii_ed_reconfig_mgmt_inst.FAMILY = "Stratix V";
defparam sdi_ii_ed_reconfig_mgmt_inst.VIDEO_STANDARD = "tr";
defparam sdi_ii_ed_reconfig_mgmt_inst.XCVR_TX_PLL_SEL = 1;
defparam sdi_ii_ed_reconfig_mgmt_inst.DIRECTION = "tx";

sample_system u0 (
 .clk_clk                      (clk_125_p),                      
 .reset_reset                  (!cpu_resetn),                  
 
 .busy_reconfig_busy           (reconfig_busy),           
 
 .mgmt_address                 (reconfig_mgmt_address[8:2]),                 
 .mgmt_read                    (reconfig_mgmt_read),                    
 .mgmt_readdata                (reconfig_mgmt_readdata),               
 .mgmt_waitrequest             (reconfig_mgmt_waitrequest),             
 .mgmt_write                   (reconfig_mgmt_write),                   
 .mgmt_writedata               (reconfig_mgmt_writedata),               

 .mif_address                  (reconfig_mif_address),                  
 .mif_read                     (reconfig_mif_read),                     
 .mif_readdata                 (reconfig_mif_readdata),                
 .mif_waitrequest              (reconfig_mif_waitrequest),             

 .to_xcvr_reconfig_to_xcvr     (reconfig_to_xcvr),     			
 .from_xcvr_reconfig_from_xcvr (reconfig_from_xcvr)  
 );

endmodule
